﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Xml;

namespace Habbo_Clothes_Downloader
{
    class Program
    {
        private static Dictionary<string, string> data;

        private static string CurrentDirectory;
        private static string HotelVersion;
        private static string Hotel;
        private static string GordonDirectory;
        private static string figuremap;
        private static string figuremapfile;
        private static string figuredata;
        private static string DownloadDirectory;

        static void Main(string[] args)
        {

            Console.Title = "Habbo Clothes Downloader";
            Console.WindowHeight = Console.LargestWindowHeight - 40;

            CurrentDirectory = Environment.CurrentDirectory;


            data = new Dictionary<string, string>();

            if (!File.Exists("config.ini"))
            {
                Console.WriteLine("Unable find config file! \n");
            }
            else
            {
                try
                {
                    using (StreamReader streamReader = new StreamReader("config.ini"))
                    {
                        string str;
                        while ((str = streamReader.ReadLine()) != null)
                        {
                            if (str.Length >= 1 && !str.StartsWith("#"))
                            {
                                int num = str.IndexOf('=');
                                if (num != -1)
                                {
                                    string key = str.Substring(0, num);
                                    string value = str.Substring(num + 1);
                                    data.Add(key, value);
                                }
                            }
                        }
                        streamReader.Close();
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("Could not process configuration file: " + ex.Message);
                }
            }

            if (data.ContainsKey("HotelVersion"))
                HotelVersion = data["HotelVersion"];
            else
                HotelVersion = "RELEASE63-201312201124-496245659";

            if (data.ContainsKey("Hotel"))
                Hotel = data["Hotel"];
            else
                Hotel = "com";

            if (data.ContainsKey("GordonDirectory"))
                GordonDirectory = data["GordonDirectory"];
            else
                GordonDirectory = "http://habboo-a.akamaihd.net/gordon/";

            if (data.ContainsKey("DownloadDirectory"))
                DownloadDirectory = data["DownloadDirectory"];
            else
                DownloadDirectory = CurrentDirectory + @"\clothes\";

            figuremap = GordonDirectory + HotelVersion + "/figuremap.xml";
            figuredata = "http://habbo." + Hotel + "/gamedata/figuredata";

            if (!Directory.Exists(DownloadDirectory))
            {
                Console.WriteLine("Directory does not exist, creating the directory!" + "\n");
                Directory.CreateDirectory(DownloadDirectory);
            }

            if (!Directory.Exists(DownloadDirectory))
            {
                Console.WriteLine("Directory does not exist, creating the directory!" + "\n");
                Directory.CreateDirectory(DownloadDirectory);
            }

            WebClient WebClient = new WebClient();

            WebClient.DownloadFile(figuremap, DownloadDirectory + @"\figuremap.xml");
            Console.WriteLine("Downloaded figuremap.xml\n");
            WebClient.DownloadFile(figuredata, DownloadDirectory + @"\figuredata.xml");
            Console.WriteLine("Downloaded figuredata.xml\n");

            figuremapfile = DownloadDirectory + @"\figuremap.xml";

            Console.WriteLine("Reading figuremap.xml\n");
            Stream Stream = WebClient.OpenRead(figuremap);
            Console.WriteLine("Completed reading figuremap --- Now downloading clothes!\n");
            StreamReader Reader = new StreamReader(Stream);

            string file = File.ReadAllText(figuremapfile);

            StringBuilder sb = new StringBuilder();
            using (StreamReader sr = new StreamReader(figuremapfile))
            {
                String line;
                while ((line = sr.ReadLine()) != null)
                {
                    sb.AppendLine(line);
                }
            }
            string allines = sb.ToString();

            StringBuilder output = new StringBuilder();
            int DownloadCount = 0;

            List<string> files = new List<string>();

            using (XmlReader reader = XmlReader.Create(new StringReader(allines)))
            {
                while (reader.Read())
                {
                    if (reader.IsStartElement())
                    {
                        switch (reader.Name)
                        {
                            case "map":
                                break;
                            case "lib":
                                string id = reader["id"];
                                if (id != null)
                                {
                                    if (!File.Exists(DownloadDirectory + id + ".swf"))
                                    {
                                        files.Add(id);
                                    }
                                }
                                break;
                        }
                    }
                }
            }

            Console.WriteLine(files.Count + " files found and needed to download");

            int onnow = 0;

            System.Threading.Thread t = new System.Threading.Thread(() =>
            {
                while (onnow < files.Count)
                {
                    System.Threading.Thread.Sleep(10000);
                    try
                    {
                        string id = files[onnow];
                        onnow++;
                        if (!File.Exists(DownloadDirectory + id + ".swf"))
                        {
                            Console.WriteLine("[" + DownloadCount + "] Downloading: " + id);
                            WebClient WebClient1 = new WebClient();
                            WebClient1.DownloadFileAsync(new Uri(GordonDirectory + HotelVersion + "/" + id + ".swf"), DownloadDirectory + id + ".swf");
                            Console.WriteLine("[" + DownloadCount + "] Downloaded: " + id);
                            DownloadCount++;
                            
                        }
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.ToString());
                    }
                }
            });
            t.Start();

            System.Threading.Thread t2 = new System.Threading.Thread(() =>
            {
                while (onnow < files.Count)
                {
                    try
                    {
                        string id = files[onnow];
                        onnow++;
                        if (!File.Exists(DownloadDirectory + id + ".swf"))
                        {
                            Console.WriteLine("[" + DownloadCount + "] Downloading: " + id);
                            WebClient WebClient1 = new WebClient();
                            WebClient1.DownloadFileAsync(new Uri(GordonDirectory + HotelVersion + "/" + id + ".swf"), DownloadDirectory + id + ".swf");
                            Console.WriteLine("[" + DownloadCount + "] Downloaded: " + id);
                            DownloadCount++;
                            
                        }
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.ToString());
                    }
                }
            });
            t2.Start();

            System.Threading.Thread t3 = new System.Threading.Thread(() =>
            {
                while (onnow < files.Count)
                {
                    try
                    {
                        string id = files[onnow];
                        onnow++;
                        if (!File.Exists(DownloadDirectory + id + ".swf"))
                        {
                            Console.WriteLine("[" + DownloadCount + "] Downloading: " + id);
                            WebClient WebClient1 = new WebClient();
                            WebClient1.DownloadFileAsync(new Uri(GordonDirectory + HotelVersion + "/" + id + ".swf"), DownloadDirectory + id + ".swf");
                            Console.WriteLine("[" + DownloadCount + "] Downloaded: " + id);
                            DownloadCount++;
                            
                        }
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.ToString());
                    }
                }
            });
            t3.Start();

            System.Threading.Thread t5 = new System.Threading.Thread(() =>
            {
                while (onnow < files.Count)
                {
                    try
                    {
                        string id = files[onnow];
                        onnow++;
                        if (!File.Exists(DownloadDirectory + id + ".swf"))
                        {
                            Console.WriteLine("[" + DownloadCount + "] Downloading: " + id);
                            WebClient WebClient1 = new WebClient();
                            WebClient1.DownloadFileAsync(new Uri(GordonDirectory + HotelVersion + "/" + id + ".swf"), DownloadDirectory + id + ".swf");
                            Console.WriteLine("[" + DownloadCount + "] Downloaded: " + id);
                            DownloadCount++;
                            
                        }
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.ToString());
                    }
                }
            });
            t5.Start();

            bool finished = false;

            System.Threading.Thread t4 = new System.Threading.Thread(() =>
            {
                while (!finished)
                {
                    if (files.Count <= DownloadCount)
                    {
                        finished = true;
                        if (DownloadCount > 0)
                        {
                            Console.WriteLine("Downloaded " + DownloadCount + " new clothes!");
                        }
                        else
                        {
                            Console.WriteLine("You have the latest clothes!");
                        }

                        

                        Console.WriteLine("Press any key to exit...");
                        Console.ReadKey(true);
                    }
                }
            });
            t4.Start();
        }
    }

}